@@ -1,12 +1,8 @@
 ##
 # GUTS R Definitions.
-# soeren.vogel@uzh.ch, carlo.albert@eawag.ch
+# soeren.vogel@uzh.ch, carlo.albert@eawag.ch, oliver.jakoby@rifcon.de, alexander.singer@rifcon.de
 # License GPL-2
-# 2015-06-25
-#
-
-
-
+# 2019-01-29
 
 
 ##
@@ -84,8 +80,7 @@ guts_setup <- function(C, Ct, y, yt, dist = 'lognormal', model = 'Proper', N = 1
 		warning( 'Survivor information at time points later than the latest concentration time point are disregarded.' )
 	}
 
-
-	#
+  #
 	# Check dist and model.
 	# Set experiment code.
 	# Set par_pos.
@@ -96,6 +91,13 @@ guts_setup <- function(C, Ct, y, yt, dist = 'lognormal', model = 'Proper', N = 1
 	mdist  <- tolower(dist)
 	mmodel <- tolower(model)
 	par_pos <- 1:5
+	
+	if (mmodel == "sd") {
+	  mdist <- "delta"
+	  mmodel <- "proper"
+	}
+	  
+	
 	if ( mdist == 'lognormal' ) {
 		if ( mmodel == 'proper' ) {
 			experiment <- 11
@@ -107,17 +109,37 @@ guts_setup <- function(C, Ct, y, yt, dist = 'lognormal', model = 'Proper', N = 1
 			stop( 'Model must be either "Proper" or "IT".' )
 		}
 	} else if ( mdist == "delta" ) {
-		if ( mmodel == 'proper' ) {
-			experiment <- 21
-			par_pos <- 1:4
-		} else if ( mmodel == 'it' ) {
-			experiment <- 22
-			par_pos <- c(1:2, 4)
-		} else {
-			stop( 'Model must be either "Proper" or "IT".' )
-		}
+	  if ( mmodel == 'proper' ) {
+	    experiment <- 21
+	    par_pos <- 1:4
+	  } else if ( mmodel == 'it' ) {
+	    experiment <- 22
+	    par_pos <- c(1:2, 4)
+	  } else {
+	    stop( 'Model must be either "Proper" or "IT".' )
+	  }
+	} else if ( mdist == "loglogistic" ) { 
+	  if ( mmodel == 'proper' ) {
+	    experiment <- 31
+	    par_pos <- 1:5
+	  } else if ( mmodel == 'it' ) {
+	    experiment <- 32
+	    par_pos <- c(1:2, 4,5)
+	  } else {
+	    stop( 'Model must be either "Proper" or "IT".' )
+	  }
+	} else if ( mdist == "external" ) {
+	  if ( mmodel == 'proper' ) {
+	    experiment <- 41
+	    par_pos <- 1:3
+	  } else if ( mmodel == 'it' ) {
+	    experiment <- 42
+	    par_pos <- c(1:2)
+	  } else {
+	    stop( 'Model must be either "Proper" or "IT".' )
+	  }
 	} else {
-		stop( 'Distribution must be either "lognormal" or "delta".' )
+	  stop( 'Distribution must be either "lognormal", "loglogistic", "external" or "delta".' )
 	}
 	par <- rep(NA, length(par_pos))
 	wpar <- c(0, 0, .Machine$double.xmax, 0, 0)
@@ -148,6 +170,7 @@ guts_setup <- function(C, Ct, y, yt, dist = 'lognormal', model = 'Proper', N = 1
 			'M'     = M,
 			'par'   = par,
 			'S'     = rep(NA, length(yt)),
+			'D'     = rep(NA, M),
 			'LL'    =  NA
 		),
 		class      = "GUTS",
@@ -157,20 +180,15 @@ guts_setup <- function(C, Ct, y, yt, dist = 'lognormal', model = 'Proper', N = 1
 	)
 	invisible( return( ret ) )
 
-} # End of guts_setup(C, Ct, y, yt, dist = 'lognormal', model = 'Proper', N = 1000, M = 10000)
-
+} # End of guts_setup()
 
 
 
 
 ##
 # Function guts_calc_loglikelihood(...).
-# Code was generated by Rcpp::compileAttributes
-# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393
-# Edit SV 2015-05-30
-#
-guts_calc_loglikelihood <- function(gobj, par) {
-	invisible(.Call('GUTS_guts_engine', PACKAGE = 'GUTS', gobj, par))
+guts_calc_loglikelihood <- function(gobj, par, external_dist = NULL) {
+	invisible(.Call('_GUTS_guts_engine', PACKAGE = 'GUTS', gobj, par, z_dist = external_dist))
 	return(gobj[['LL']])
 }
 
@@ -181,13 +199,24 @@ guts_calc_loglikelihood <- function(gobj, par) {
 
 ##
 # Function guts_calc_survivalprobs(...).
-#
-guts_calc_survivalprobs <- function(gobj, par) {
-	invisible(.Call('GUTS_guts_engine', PACKAGE = 'GUTS', gobj, par))
+guts_calc_survivalprobs <- function(gobj, par, external_dist = NULL) {
+	invisible(.Call('_GUTS_guts_engine', PACKAGE = 'GUTS', gobj, par, z_dist = external_dist))
 	return(gobj[['S']])
 }
 
 
+##
+# Function guts_report_damage(...).
+guts_report_damage <- function(gobj) {
+  return(
+    data.frame(
+      time = seq(min(gobj[['yt']]), max(gobj[['yt']]), length.out = gobj[['M']]),
+      damage = gobj[['D']]
+      )
+  )
+}
+
+
 
 
 
@@ -251,6 +280,15 @@ guts_calc_survivalprobs <- function(gobj, par) {
 		cat( "\n", sep="" )
 	}
 
+	# Damage
+	prf <- paste("Damage (n=", length(object$D), "; subset: 30 regularly spaced values)", sep="")
+	if ( length(object$D) > 0 ) {
+		prf <- paste(prf, ": ", sep="")
+		cat( .g_print_help(object$D[seq(1, length(object$D), length.out = 30)], width, digits, exdent = 2, prefix = prf), sep="\n" )
+	} else {
+		cat( "\n", sep="" )
+	}
+
 	# Loglikelihood
 	cat( "Loglikelihood: ", object$LL, "\n", sep="" )
 
@@ -309,39 +347,3 @@ print.GUTS <- function(x, ...) {
 "mostattributes<-.GUTS" <- function(x, which, value) {
 	stop( "Use function `guts_setup()` for changing attributes of a GUTS object." )
 }
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
